"use strict";

var _interopRequireWildcard = require("@babel/runtime-corejs2/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _Object$defineProperty = require("@babel/runtime-corejs2/core-js/object/define-property");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = ResolverFloat;

var _parseFloat2 = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/parse-float"));

var _ducky = _interopRequireDefault(require("ducky"));

var GraphQL = _interopRequireWildcard(require("graphql"));

var GraphQLLanguage = _interopRequireWildcard(require("graphql/language"));

var _error = require("graphql/error");

/*
**  GraphQL-Tools-Types -- Custom Scalar Types for GraphQL-Tools
**  Copyright (c) 2016-2019 Dr. Ralf S. Engelschall <rse@engelschall.com>
**
**  Permission is hereby granted, free of charge, to any person obtaining
**  a copy of this software and associated documentation files (the
**  "Software"), to deal in the Software without restriction, including
**  without limitation the rights to use, copy, modify, merge, publish,
**  distribute, sublicense, and/or sell copies of the Software, and to
**  permit persons to whom the Software is furnished to do so, subject to
**  the following conditions:
**
**  The above copyright notice and this permission notice shall be included
**  in all copies or substantial portions of the Software.
**
**  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
**  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
**  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
**  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
**  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
**  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
**  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/*  external requirements  */
function ResolverFloat(options = {}) {
  const errors = [];
  if (!_ducky.default.validate(options, "{ name: string, min?: number, max?: number, fn?: function }", errors)) throw new _error.GraphQLError("[graphql-tools-types] " + `invalid parameters: ${errors.join("; ")}`, []);

  const validate = (value, ast = null) => {
    if (options.min !== undefined && value < options.min) throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + `minimum value is ${options.min}`, ast !== null ? [ast] : []);
    if (options.max !== undefined && value > options.max) throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + `maximum value is ${options.max}`, ast !== null ? [ast] : []);
    if (options.fn !== undefined && !options.fn(value)) throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + "value not valid", ast !== null ? [ast] : []);
  };

  return {
    /*  serialize value sent as output to the client  */
    __serialize: value => {
      return value;
    },

    /*  parse value received as input from client  */
    __parseValue: value => {
      if (typeof value === "string") {
        const re = /^[+-]?(?:\d*\.?\d+(?:[eE][+-]?\d+)?|Infinity)$/;
        if (!re.test(value)) throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + `invalid input value (matching regular expression "${re}" expected)`, []);
        value = (0, _parseFloat2.default)(value);
      } else if (typeof value !== "number") throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + "invalid input value (type \"number\" or \"string\" expected)", []);

      validate(value);
      return value;
    },

    /*  parse value received as literal in AST  */
    __parseLiteral: ast => {
      if (ast.kind !== GraphQLLanguage.Kind.FLOAT) throw new _error.GraphQLError(`[graphql-tools-types] ${options.name}: ` + "invalid AST node (kind \"FLOAT\" expected)", [ast]);
      const value = GraphQL.GraphQLFloat.parseLiteral(ast);
      validate(value, ast);
      return value;
    }
  };
}
